	'-----------------------------------------------------------'
	'USER DEFINITIONS (DEFINE HERE)
	'
	'Text editor w/ absolute path
	'ex: "C:\ATA\Local\pfe32\PFE32.EXE"	
	Const TEXT_EDITOR = "%SystemRoot%\system32\NOTEPAD.EXE"
	'
	'Absolute path to folder that will contain the file, APPEND SLASH
	Const TEMP_FOLDER = "S:\scratch\"
	'
	'Group to preview
	' True  = Active group
	' False = Select group from interactive list
	Const ACTIVE_GROUP = True
	'
	'Field format
	' 0 = Small Field
	' 1 = Large Field (CSys, Material, Prop)
	' 2 = Large Field (All but Elements)
	' 3 = Large Field
	' 4 = Large Field (CSys, Node)
	Const FIELD_FORMAT = 4
	'-----------------------------------------------------------'

' Name: PrevGrpBlk.BAS
' Author: Ryan Blelloch
' Support: kurt.knutson@ata-e.com
'
' Version #: 02
'
' Version Description: Created. User defines text editor, temp file location, and group selection
'                                     by editing values in this file. Exports files to temp location and attempts
'                                     to open them with defined program and file as cmd line arg.
'
' Date Modified: 20170707
'
' Bug Fixes:
'		- File paths with spaces are now handled correctly
'       - Failed Shell calls are now handled correctly
'       - Default text editor is now C:\ATA\Local\pfe32\PFE32.EXE since it is most common on ATA computers
'
' Date Modified: 20170712
'
' Version 02:
'		- Moved user constants to the top of the file
'		- Ensured RBE CTEs would be included in bulk data file
'
' Date Modified: 20170720
'
'
'Copyright (c) <2017> <ATA Engineering, Inc.>
'
'Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
'associated documentation files (the "Software") to use, copy, modify, merge, and/or distribute
'royalty-free copies of the Software, and to permit persons to whom the Software is furnished
'to do so, subject to the following conditions:
'The above copyright notice and this permission notice shall be included in all copies or
'substantial portions of the Software.
'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
'INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
'FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
'IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
'WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
'OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


Sub Main

	'FEMAP objects
	'get active femap model
    Dim App As femap.model
    Set App = feFemap()

	'selected user group
    Dim blkGrp As femap.Group
    Set blkGrp = App.feGroup

    'temp analysis set for exporting group bulk data
    Dim tmpAnalysis As femap.AnalysisMgr
    Set tmpAnalysis = App.feAnalysisMgr

    'name of group bulk data file
    Dim fileRoot As String

    'full path and name of group bulk data file
    Dim absolutePath As String
    ' Used to change absolutePath to something does not already exist
    Dim updatedFilename As String
    Dim fileNum As Long

    'System call to open up text editor
    Dim openFileWithTextEditor As String

    'task ID of text editor shell call
    Dim taskID As Long
    taskID = -1

    'Femap return codes
    Dim rc As femap.zReturnCode

    'Prompt User with User Definition Settings
    App.feAppMessage( FCM_NORMAL, "-----Preview Group Bulk Data-----" )
    App.feAppMessage( FCM_NORMAL, "-(Change settings by editing file)-" )
    App.feAppMessage( FCM_NORMAL, "Text Editor:       " + CStr( TEXT_EDITOR ) )
    App.feAppMessage( FCM_NORMAL, "Output Folder: " + CStr( TEMP_FOLDER ) )

    'get group
    If( ACTIVE_GROUP) Then
    	blkGrp.Get( blkGrp.Active )
    Else
    	'user selects group
    	rc = blkGrp.SelectID( "Select group to preview" )
    	If( rc <> femap.zReturnCode.FE_OK) Then
    		Exit Sub
    	End If
    End If

    'ensure a group was selected
    If( blkGrp.ID < 1) Then
    	App.feAppMessage( FCM_WARNING, "No group selected" )
    	Exit Sub
    End If

    'Create dummy analysis set manager
    tmpAnalysis.ID = tmpAnalysis.NextEmptyID
    tmpAnalysis.AnalysisType = FAT_STATIC
    tmpAnalysis.NasBulkRigidThermalExp = True
    tmpAnalysis.NasExecSkipStandard = True
    tmpAnalysis.NasBulkSkipStandard = False
    tmpAnalysis.SkipStandard = True
    tmpAnalysis.Solver = FAM_NX_NASTRAN
    tmpAnalysis.NasBulkGroupID = blkGrp.ID
    tmpAnalysis.NasBulkLargeField = FIELD_FORMAT
    tmpAnalysis.title = App.ModelName + "_" + CStr(blkGrp.ID)
    tmpAnalysis.Put(tmpAnalysis.ID)
    'make sure newly created analysis is active for export
    tmpAnalysis.Active = tmpAnalysis.ID

    'write out bulk data to file using dummy analysis manager
    fileRoot = tmpAnalysis.title
    'remove bad characters from filename
    fixFileRoot(fileRoot)
    absolutePath = TEMP_FOLDER + fileRoot
    fileNum = 0

    'change filename until it is not a preexisting file
    updatedFilename =absolutePath + ".preview"
    While (FileExists(updatedFilename))
    	fileNum = fileNum+ 1
    	updatedFilename =absolutePath
    	updatedFilename = updatedFilename + " (" + CStr(fileNum) + ").preview"
    Wend
    absolutePath = updatedFilename

    'Attempt to export bulk data to file
    rc = App.feFileWriteNastran( 8, absolutePath )

    'Export failed, so clean up analysis set and exit
    If( rc <> FE_OK) Then
    	App.feAppMessage(FCM_ERROR, "Exporting preview to " + absolutePath + " failed")
    	deleteAnalysisSet(App, tmpAnalysis)
    	Exit Sub
    End If

    'Prompt User with User Definition Settings
    App.feAppMessage( FCM_NORMAL, "-----Preview Group Bulk Data-----" )
    App.feAppMessage( FCM_NORMAL, "-(Change settings by editing file)-" )
    App.feAppMessage( FCM_NORMAL, "Text Editor:       " + CStr( TEXT_EDITOR ) )
    App.feAppMessage( FCM_NORMAL, "Output Folder: " + CStr( TEMP_FOLDER ) )

    'If export was a success, attempt to open the preview file with the defined text editor
    openFileWithTextEditor = """" + TEXT_EDITOR + """ """ +absolutePath + """"
    On Error Resume Next
    taskID = Shell(openFileWithTextEditor, vbNormalFocus)

    'Make sure the preview opened correctly
    If( taskID < 0 ) Then
    	App.feAppMessage(FCM_ERROR, "Failed to open preview of bulk data file")
    	App.feAppMessage(FCM_ERROR, "Attempted to run: " + openFileWithTextEditor)

    Else
    	'Inform user where the file has been made
    	App.feAppMessage(FCM_NORMAL, "Preview file created: " +absolutePath)
    End If

    'Clean up dummy analysis set
    deleteAnalysisSet(App, tmpAnalysis)
End Sub

'Check if the file exists. If a file exists but can't be opened, it will assume the file does not exist.
Public Function FileExists( ByRef absolutePath As String ) As Boolean

	Dim testFile As Object
	Set testFile = CreateObject("Scripting.FileSystemObject")
	FileExists = testFile.FileExists(absolutePath)

End Function

'Thorough deletion of the dummy analysis set. If it fails, prompts user to manually delete the file
Public Sub deleteAnalysisSet(ByRef App As femap.model, ByRef tmpAnalysis As femap.AnalysisMgr)

	rc = tmpAnalysis.Deletable( tmpAnalysis.ID )
    If( rc = femap.zReturnCode.FE_OK ) Then
    	rc = tmpAnalysis.Delete( tmpAnalysis.ID )
    End If

	'If the analysis set is unable to be deleted, prompt the user to delete it themselves
	If(rc <> femap.zReturnCode.FE_OK) Then
		    	App.feAppMessage( FCM_WARNING, _
		    		"Unable to delete dummy analysis set: " + CStr(tmpAnalysis.ID) + ". " + tmpAnalysis.title)
	End If

End Sub

Public Sub fixFileRoot(ByRef fRoot As String)

	Dim replaceChar As String
	replaceChar = "_"

	fRoot = Replace( fRoot, "\", replaceChar,1, -1)
	fRoot = Replace( fRoot, "/", replaceChar,1, -1)
	fRoot = Replace( fRoot, ":", replaceChar,1, -1)
	fRoot = Replace( fRoot, "~", replaceChar,1, -1)
	fRoot = Replace( fRoot, "#", replaceChar,1, -1)
	fRoot = Replace( fRoot, "%", replaceChar,1, -1)
	fRoot = Replace( fRoot, "&", replaceChar,1, -1)
	fRoot = Replace( fRoot, "*", replaceChar,1, -1)
	fRoot = Replace( fRoot, "{", replaceChar,1, -1)
	fRoot = Replace( fRoot, "}", replaceChar,1, -1)
	fRoot = Replace( fRoot, "<", replaceChar,1, -1)
	fRoot = Replace( fRoot, ">", replaceChar,1, -1)
	fRoot = Replace( fRoot, "?", replaceChar,1, -1)
	fRoot = Replace( fRoot, "+", replaceChar,1, -1)
	fRoot = Replace( fRoot, "|", replaceChar,1, -1)
	fRoot = Replace( fRoot, """", replaceChar,1, -1)

End Sub
